"""
show_aio.py defines functions showing aio device info.
"""

from Tkinter import *
import thread
import time


class show_aio (Tk):
    """
    GUI class for aio devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Analog I/O : ' + str(self.__device_index))
	# set layouts of widgets
	self.__digital = []
	for idx in range(len(self.__player.aio[self.__device_index])):
	    Label(self, text='#%d : ' % idx).grid(row=idx, column=0, sticky=E)
	    self.__digital.append(Label(self))
	    self.__digital[idx].grid(row=idx, column=1, sticky=W)
	self.grid()
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __binary_string(self, value):
        """
	convert an integer value to a binary string.
	"""
	binstr = ''
	for idx in range(8):
	    binstr = str(value & 0x01) + binstr
	    value >>= 1
	return binstr


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a aio update thread>'
	while self.__update_flag:
	    try:
		# update digital info
		for idx in range(len(self.__player.aio[self.__device_index])):
		    self.__digital[idx].configure(text='%s' %
			self.__player.aio[self.__device_index][idx])
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<aio:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the aio update thread finishes>'

