"""
show_bumper.py defines functions showing bumper device info.
"""

from Tkinter import *
import thread
import time
import math


class show_bumper (Tk):
    """
    GUI class for bumper devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Bumper : ' + str(self.__device_index))
	self.__win = Canvas(self, bg='black', width=200, height=200)
	# set layouts of widgets
	self.__win.grid(row=0, column=0, sticky=N+E+S+W)
	self.grid()
	# draw equidistance circles
	for idx in range(20,142,20):
	    self.__win.create_oval(100-idx, 100-idx, 100+idx, 100+idx, outline='gray40')
	# retrieve the bumper geometry
	geom = self.__player.get_bumper_geometry()
	# generate oval items
	self.__bumpers = []
	for idx in range(len(geom)):
	    x = 100 - geom[idx][1] * 20 / 100.0
	    y = 100 - geom[idx][0] * 20 / 100.0
	    self.__bumpers.append(self.__win.create_oval(x-5,y-5, x+5,y+5,
	    				fill='green', outline='green'))
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a bumper update thread>'
	while self.__update_flag:
	    try:
		# update canvas window
		for idx in range(len(self.__player.bumper[self.__device_index])):
		    if self.__player.bumper[self.__device_index][idx]:
		        self.__win.itemconfig(self.__bumpers[idx], fill='red', outline='red')
		    else:
		        self.__win.itemconfig(self.__bumpers[idx], fill='green', outline='green')
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<bumper:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the bumper update thread finishes>'

