"""
show_localization.py defines functions showing localization device info.
"""

from Tkinter import *
import thread
import time
import math


class show_localization (Tk):
    """
    GUI class for localization devices.
    """

    def __init__(self, player, index=0, speed=5, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Localization : ' + str(self.__device_index))
	# compute the proper size of the GUI window
	self.__width, self.__height, self.__ppm = self.__player.get_map_size(1)
	self.__scale = int(self.__width/200)
	self.__width, self.__height, self.__ppm = self.__player.get_map_size(self.__scale)
	self.__win = Canvas(self, bg='black', width=self.__width, height=self.__height)
	# set layouts of widgets
	self.__win.grid(row=0, column=0, sticky=N+E+S+W)
	self.grid()
	# generate buckets for color blobs
	self.__oval = []
	self.__center = []
	for idx in range(10):
	    self.__oval.append(self.__win.create_oval(0,0,0,0,outline='yellow'))
	    self.__center.append(self.__win.create_oval(0,0,0,0,fill='yellow',outline='yellow'))
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __int2color(self, color):
        """
	convert an integer into a color string.
	"""
	return '#%02x%02x%02x' % (color>>16, (color&0x00FF00)>>8, (color&0x0000FF))


    def __eigen(self, cm):
        """
	compute the eigen values and vectors of a given covariance matrix.
	"""
	s = math.sqrt(cm[0][0]*cm[0][0] - 2*cm[0][0]*cm[1][1] +
		      cm[1][1]*cm[1][1] + 4*cm[0][1]*cm[0][1]);
	values = (0.5 * (cm[0][0] + cm[1][1] + s), 0.5 * (cm[0][0] + cm[1][1] - s))
	vectors = ((-0.5 * (-cm[0][0] + cm[1][1] - s), -0.5 * (-cm[0][0] + cm[1][1] + s)),
		   (cm[0][1], cm[0][1]))
	return (values, vectors)


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a localization update thread>'
	while self.__update_flag:
	    try:
	    # update each hypothesis
		for idx in range(len(self.__player.localization[self.__device_index])):
		    # draw the center point
		    cx, cy, ca = self.__player.localization[self.__device_index][idx][0]
		    cx = cx / 2000.0 * self.__ppm * self.__scale;
		    cy = self.__height - (cy / 2000.0 * self.__ppm * self.__scale);
		    self.__win.coords(self.__center[idx], cx-2,cy-2, cx+2,cy+2)
		    # draw the ellipsoid
		    #
		    # no oval rotation in Tk ???
		    #
#		    eval, evec = self.__eigen(self.__player.localization[self.__device_index][idx][1])
#		    oa = math.atan2(evec[1][0], evec[0][0])
#		    sx = (3*math.sqrt(abs(eval[0]))) / 2000.0 * self.__ppm * self.__scale
##		    sy = (3*math.sqrt(abs(eval[1]))) / 2000.0 * self.__ppm * self.__scale
#		    self.__win.coords(self.__oval[idx], cx-sx,cy-sy, cx+sx,cy+sy)
		# hide extra hypothesis objects
		for idx in range(len(self.__player.localization[self.__device_index]),10):
		    self.__win.coords(self.__center[idx], 0,0, 0,0)
		    self.__win.coords(self.__oval[idx], 0,0, 0,0)
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<localization:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the localization update thread finishes>'

